window.Gleam.initSearch([{"doc":"pika_id","title":"pika_id","content":"# pika\n\n&gt; Gleam implementation of Pika\n\nCombine Stripe IDs with Snowflakes you get Pika! The last ID system you&#39;ll ever need!\nCombining pragmatism with functionality\n\n## Features\n\n- Written using only official Gleam libs.\n- Different client implementations.\n\n## Installation\n\nAdd `pika_id` to your Gleam project.\n\n```sh\ngleam add pika_id\n```\n\n## Basic Usage\n\n```gleam\nimport pika_id.{Prefix}\n\npub fn main() {\n  let assert Ok(pika) =\n    pika_id.pika_init([\n      Prefix(name: &quot;user&quot;, description: &quot;User IDs&quot;, secure: False),\n      Prefix(name: &quot;post&quot;, description: &quot;Post IDs&quot;, secure: False),\n      Prefix(name: &quot;sk&quot;, description: &quot;Secret Keys&quot;, secure: True),\n    ])\n  // -&gt; Pika\n\n  pika.gen(&quot;user&quot;)\n  // -&gt; Ok(&quot;user_MzMyNTUyMDY3Mzc0MDcxODA4&quot;)\n\n  pika.gen(&quot;sk&quot;)\n  // -&gt; Ok(&quot;sk_c19iOTJlMDVmZDU2NTY3YzA1MTBmOTBkOTZjOTdmNzgyNl8zMzI1NTIwNjczOTA4NDkwMjQ&quot;)\n\n  pika\n  |&gt; pika_id.pika_deconstruct(\n    &quot;sk_c185NTBlNTI0YTc0NjZkNjc2NmVlMDdhMDBiYjliZTMyYV8zMzI1NTIzMDU0MzQzNzgyNDA&quot;,\n  )\n  // -&gt;\n  //   Ok(\n  //     PikaIdMetadata(\n  //       &quot;sk_c185NTBlNTI0YTc0NjZkNjc2NmVlMDdhMDBiYjliZTMyYV8zMzI1NTIzMDU0MzQzNzgyNDA&quot;,\n  //       &quot;sk&quot;,\n  //       &quot;Secret Keys&quot;,\n  //       True,\n  //       &quot;c185NTBlNTI0YTc0NjZkNjc2NmVlMDdhMDBiYjliZTMyYV8zMzI1NTIzMDU0MzQzNzgyNDA&quot;,\n  //       &quot;s_950e524a7466d6766ee07a00bb9be32a_332552305434378240&quot;,\n  //       332552305434378240,\n  //       1720281848138,\n  //       1640995200000,\n  //       628,\n  //       0\n  //     )\n  //   )\n}\n```\n\n## Node IDs\n\nBy default, Node IDs are calculated by finding the MAC address of the first public network interface device, then calculating the modulo against 1024.\n\nThis works well for smaller systems, but if you have a lot of nodes generating Snowflakes, then collision is possible. In this case, you should create an internal singleton service which keeps a rolling count of the assigned node IDs - from 1 to 1023. Then, services that generate Pikas should call this service to get assigned a node ID.\n\nYou can then pass in the node ID when initializing Pika like this:\n\n```gleam\n  let assert Ok(pika) =\n    pika_custom_init(\n      [\n        Prefix(name: &quot;user&quot;, description: &quot;User IDs&quot;, secure: False),\n        Prefix(name: &quot;post&quot;, description: &quot;Post IDs&quot;, secure: False),\n        Prefix(name: &quot;sk&quot;, description: &quot;Secret Keys&quot;, secure: True),\n      ],\n      [WithNodeId(628)],\n      // You can also specify Epoch: [WithNodeId(628), WithEpoch(1_000_000_000)]\n    )\n```\n","url":"index.html"},{"doc":"pika_id/utils","title":"Address","content":"pub type Address {\n  Hwaddr(List(Int))\n  Flags\n  Addr\n  Netmask\n  Broadaddr\n}\n\nHwaddr(List(Int))\n\n\nFlags\n\n\nAddr\n\n\nNetmask\n\n\nBroadaddr\n\n\nSynonyms:\npika_id/utils.Address\npika_id/utils Address","url":"pika_id/utils.html#Address"},{"doc":"pika_id/utils","title":"Interface","content":"pub type Interface =\n  #(List(Int), List(Address))\n\n\nSynonyms:\npika_id/utils.Interface\npika_id/utils Interface","url":"pika_id/utils.html#Interface"},{"doc":"pika_id/utils","title":"compute_node_id","content":"pub fn compute_node_id() -&gt; Int\n Compute the Node ID for the Snowflake gen.\nSynonyms:\npika_id/utils.compute_node_id\npika_id/utils compute_node_id","url":"pika_id/utils.html#compute_node_id"},{"doc":"pika_id/utils","title":"get_mac_address","content":"pub fn get_mac_address() -&gt; String\n Get the mac address of the first available public interface on the device.\nSynonyms:\npika_id/utils.get_mac_address\npika_id/utils get_mac_address","url":"pika_id/utils.html#get_mac_address"},{"doc":"pika_id/utils","title":"get_timestamp","content":"pub fn get_timestamp() -&gt; Int\n Get timestamp in millisec.\nSynonyms:\npika_id/utils.get_timestamp\npika_id/utils get_timestamp","url":"pika_id/utils.html#get_timestamp"},{"doc":"pika_id/utils","title":"getifaddrs","content":"pub fn getifaddrs() -&gt; Result(\n  List(#(List(Int), List(Address))),\n  String,\n)\n\nSynonyms:\npika_id/utils.getifaddrs\npika_id/utils getifaddrs","url":"pika_id/utils.html#getifaddrs"},{"doc":"pika_id/utils","title":"is_alphanumeric_list","content":"pub fn is_alphanumeric_list(\n  str_list: List(String),\n) -&gt; Result(String, Nil)\n\nSynonyms:\npika_id/utils.is_alphanumeric_list\npika_id/utils is_alphanumeric_list","url":"pika_id/utils.html#is_alphanumeric_list"},{"doc":"pika_id/utils","title":"pika_id/utils","content":"","url":"pika_id/utils.html"},{"doc":"pika_id/snowflake","title":"DeconstructedSnowflake","content":"pub type DeconstructedSnowflake {\n  DeconstructedSnowflake(\n    id: Int,\n    timestamp: Int,\n    epoch: Int,\n    node_id: Int,\n    seq: Int,\n  )\n}\n The deconstructed Snowflake ID content.\nDeconstructedSnowflake(\n  id: Int,\n  timestamp: Int,\n  epoch: Int,\n  node_id: Int,\n  seq: Int,\n)\n\n\nSynonyms:\npika_id/snowflake.DeconstructedSnowflake\npika_id/snowflake DeconstructedSnowflake","url":"pika_id/snowflake.html#DeconstructedSnowflake"},{"doc":"pika_id/snowflake","title":"Message","content":"pub opaque type Message(snowflake)\n The messages that the generator can receive.\n\nSynonyms:\npika_id/snowflake.Message\npika_id/snowflake Message","url":"pika_id/snowflake.html#Message"},{"doc":"pika_id/snowflake","title":"Snowflake","content":"pub opaque type Snowflake\n The Snowflake generator content.\n Holds the state of the gen and is used to generate IDs.\n\nSynonyms:\npika_id/snowflake.Snowflake\npika_id/snowflake Snowflake","url":"pika_id/snowflake.html#Snowflake"},{"doc":"pika_id/snowflake","title":"SnowflakeGen","content":"pub opaque type SnowflakeGen\n Type of the generator.\n It holds the actor subject that is used to handle the generator state.\n \n # Example\n \n import pika_id/snowflake.{create_snowflake, start_snowflake}\n\n pub fn main() {\n   let assert Ok(snowflake_gen) =\n     create_snowflake()\n     |&gt; start_snowflake()\n \n   let id = snowflake_gen |&gt; snowflake.generate\n }\n \n\nSynonyms:\npika_id/snowflake.SnowflakeGen\npika_id/snowflake SnowflakeGen","url":"pika_id/snowflake.html#SnowflakeGen"},{"doc":"pika_id/snowflake","title":"SnowflakeId","content":"pub type SnowflakeId =\n  Int\n Type alias for the Snowflake ID\n\nSynonyms:\npika_id/snowflake.SnowflakeId\npika_id/snowflake SnowflakeId","url":"pika_id/snowflake.html#SnowflakeId"},{"doc":"pika_id/snowflake","title":"default_epoch","content":"pub const default_epoch: Int\n The default epoch for the generator.\nSynonyms:\npika_id/snowflake.default_epoch\npika_id/snowflake default_epoch","url":"pika_id/snowflake.html#default_epoch"},{"doc":"pika_id/snowflake","title":"max_seq","content":"pub const max_seq: Int\n The max number of seq that can be generated in a millisec.\nSynonyms:\npika_id/snowflake.max_seq\npika_id/snowflake max_seq","url":"pika_id/snowflake.html#max_seq"},{"doc":"pika_id/snowflake","title":"create_snowflake","content":"pub fn create_snowflake() -&gt; Snowflake\n Creates a new Snowflake generator with default settings.\nSynonyms:\npika_id/snowflake.create_snowflake\npika_id/snowflake create_snowflake","url":"pika_id/snowflake.html#create_snowflake"},{"doc":"pika_id/snowflake","title":"deconstruct_snowflake_id","content":"pub fn deconstruct_snowflake_id(\n  snowflake_gen: SnowflakeGen,\n  snowflake_id: Int,\n) -&gt; DeconstructedSnowflake\n Deconstruct the Snowflake ID. Recuires a Snowflake Gen for finding epoch.\n # Examples\n \n let id = snowflake_gen |&gt; snowflake.generate\n snowflake.deconstruct_snowflake_id(snowflake_gen, id)\n |&gt; io.debug \n // DeconstructedSnowflake(\n //   id, \n //   timestamp, \n //   epoch, \n //   node_id, \n //   seq\n // )\n \nSynonyms:\npika_id/snowflake.deconstruct_snowflake_id\npika_id/snowflake deconstruct_snowflake_id","url":"pika_id/snowflake.html#deconstruct_snowflake_id"},{"doc":"pika_id/snowflake","title":"generate","content":"pub fn generate(snowflake_gen: SnowflakeGen) -&gt; Int\n Generates a new Snowflake ID.\n\n # Examples\n \n import pika_id/snowflake.{\n   create_snowflake, start_snowflake, with_epoch, with_node_id,\n }\n \n pub fn main() {\n   let epoch = 1_650_153_600_000\n   let node_id = 628\n \n   let assert Ok(snowflake_gen) =\n     create_snowflake()\n     |&gt; with_epoch(epoch)\n     |&gt; with_node_id(node_id)\n     |&gt; start_snowflake()\n \n   let id = snowflake_gen |&gt; snowflake.generate\n }\n \nSynonyms:\npika_id/snowflake.generate\npika_id/snowflake generate","url":"pika_id/snowflake.html#generate"},{"doc":"pika_id/snowflake","title":"start_snowflake","content":"pub fn start_snowflake(\n  snowflake: Snowflake,\n) -&gt; Result(SnowflakeGen, String)\n Starts the generator.\nSynonyms:\npika_id/snowflake.start_snowflake\npika_id/snowflake start_snowflake","url":"pika_id/snowflake.html#start_snowflake"},{"doc":"pika_id/snowflake","title":"with_epoch","content":"pub fn with_epoch(snowflake: Snowflake, epoch: Int) -&gt; Snowflake\n Sets the epoch for the generator.\nSynonyms:\npika_id/snowflake.with_epoch\npika_id/snowflake with_epoch","url":"pika_id/snowflake.html#with_epoch"},{"doc":"pika_id/snowflake","title":"with_node_id","content":"pub fn with_node_id(\n  snowflake: Snowflake,\n  node_id: Int,\n) -&gt; Snowflake\n Sets the Node ID for the generator.\nSynonyms:\npika_id/snowflake.with_node_id\npika_id/snowflake with_node_id","url":"pika_id/snowflake.html#with_node_id"},{"doc":"pika_id/snowflake","title":"pika_id/snowflake","content":"","url":"pika_id/snowflake.html"},{"doc":"pika_id","title":"Pika","content":"pub type Pika {\n  Pika(\n    gen: fn(String) -&gt; Result(String, String),\n    deconstruct: fn(String) -&gt; Result(PikaIdMetadata, String),\n  )\n}\n Pika generator type.\nPika(\n  gen: fn(String) -&gt; Result(String, String),\n  deconstruct: fn(String) -&gt; Result(PikaIdMetadata, String),\n)\n\n\nSynonyms:\npika_id.Pika\npika_id Pika","url":"pika_id.html#Pika"},{"doc":"pika_id","title":"PikaIdMetadata","content":"pub type PikaIdMetadata {\n  PikaIdMetadata(\n    id: String,\n    prefix: String,\n    description: String,\n    secure: Bool,\n    tail: String,\n    decoded_tail: String,\n    snowflake: Int,\n    timestamp: Int,\n    epoch: Int,\n    node_id: Int,\n    seq: Int,\n  )\n}\n Deconstructed format of a Pika ID. \n Contains metas from ID&#39;s prefix and snowflake.\nPikaIdMetadata(\n  id: String,\n  prefix: String,\n  description: String,\n  secure: Bool,\n  tail: String,\n  decoded_tail: String,\n  snowflake: Int,\n  timestamp: Int,\n  epoch: Int,\n  node_id: Int,\n  seq: Int,\n)\n\n\nSynonyms:\npika_id.PikaIdMetadata\npika_id PikaIdMetadata","url":"pika_id.html#PikaIdMetadata"},{"doc":"pika_id","title":"PikaOption","content":"pub type PikaOption {\n  WithEpoch(Int)\n  WithNodeId(Int)\n}\n Options for pika ID setup.\n - WithEpoch: customize the epoch (millis) that IDs are derived from - by default, this is 1640995200000 (Jan 1 2022)\n - WithNodeId: by default, Node IDs are calculated by finding the MAC address of the first public network interface device, then calculating the modulo against 1024\nWithEpoch(Int)\n\n\nWithNodeId(Int)\n\n\nSynonyms:\npika_id.PikaOption\npika_id PikaOption","url":"pika_id.html#PikaOption"},{"doc":"pika_id","title":"Prefix","content":"pub type Prefix {\n  Prefix(name: String, description: String, secure: Bool)\n}\n When creating a pika ID, you must specify the prefix to be prepended - the general rule of thumb should be to use a different prefix for each object type (e.g. user, team, post, etc).\n Type prefixes should be lowercase, short, alphanumeric strings.\n \n ## Examples\n \n Prefix(&quot;user&quot;, &quot;User ID&quot;, False) // Correct\n Prefix(&quot;sk&quot;, &quot;Secret key&quot;, True) // Correct\n Prefix(&quot;conn_acc&quot;, &quot;ID for connected accounts&quot;, False) \n // -&gt; &quot;One of the prefixes is invalid (prefix &#39;conn_acc&#39; must be Alphanumeric)&quot;\n Prefix(&quot;pa$$word&quot;, &quot;password id here&quot;, False) \n // -&gt; &quot;One of the prefixes is invalid (prefix &#39;pa$$word&#39; must be Alphanumeric)&quot;\n \nPrefix(name: String, description: String, secure: Bool)\n\n\nSynonyms:\npika_id.Prefix\npika_id Prefix","url":"pika_id.html#Prefix"},{"doc":"pika_id","title":"pika_custom_init","content":"pub fn pika_custom_init(\n  prefixes: List(Prefix),\n  options: List(PikaOption),\n) -&gt; Result(Pika, String)\n Initialize Pika with custom Epoch and/or Node ID. \n \n To guarantee that developers use the correct pre-defined prefix types for the right object types, pika requires you to &quot;register&quot; them before they&#39;re used to prevent warnings from being thrown.\n\n This is also where you define if a prefix type should be cryptographically secure or not.\n \n If you dont need to specify options, use `pika_init` instead. (or pass [] as options).\n \n ## Examples\n \n #### Options\n \n let assert Ok(pika) =\n   pika_custom_init(\n     [\n       Prefix(&quot;user&quot;, &quot;User ID&quot;, False), \n       Prefix(&quot;sk&quot;, &quot;Secret key&quot;, True)\n     ],\n     [WithEpoch(1_000_000_000), WithNodeId(640)],\n   ) \n // -&gt; Pika\n \n \n #### No options\n \n let assert Ok(pika) = pika_custom_init([\n   Prefix(&quot;user&quot;, &quot;User ID&quot;, False), \n   Prefix(&quot;sk&quot;, &quot;Secret key&quot;, True)\n ], []) \n // -&gt; Pika\n \nSynonyms:\npika_id.pika_custom_init\npika_id pika_custom_init","url":"pika_id.html#pika_custom_init"},{"doc":"pika_id","title":"pika_deconstruct","content":"pub fn pika_deconstruct(\n  pika: Pika,\n  id: String,\n) -&gt; Result(PikaIdMetadata, String)\n Deconstruct a pika ID into metadata (`PikaIDMetadata`)\n ## Examples\n \n pika.deconstruct(&quot;user_MzMyNTAwNTIwMDQwNjExODQw&quot;)\n // -&gt; \n Ok(\n   PikaIdMetadata(\n     &quot;user_MzMyNTAwNTIwMDQwNjExODQw&quot;,\n     &quot;user&quot;,\n     &quot;User ID&quot;,\n     False,\n     &quot;MzMyNTAwNTIwMDQwNjExODQw&quot;,\n     &quot;332500520040611840&quot;,\n     332500520040611840,\n     1720269501538,\n     1640995200000,\n     628,\n     0\n   )\n )\n \nSynonyms:\npika_id.pika_deconstruct\npika_id pika_deconstruct","url":"pika_id.html#pika_deconstruct"},{"doc":"pika_id","title":"pika_gen","content":"pub fn pika_gen(\n  pika: Pika,\n  prefix: String,\n) -&gt; Result(String, String)\n Generate a pika ID. \n Tail will be generated based on prefix secure status.\n \n Example of a normal decoded tail: `129688685040889861`\n \n Example of a cryptographically secure decoded tail: `s_387d0775128c383fa8fbf5fd9863b84aba216bcc6872a877_129688685040889861`\n \n ## Examples\n \n \n pika.gen(&quot;user&quot;)\n // -&gt; Ok(&quot;user_MzMyNTAwNTIwMDUzMTk0NzUy&quot;)\n \n \n \n pika |&gt; pika_gen(&quot;user&quot;)\n // -&gt; Ok(&quot;user_MzMyNTAwNTIwMDQwNjExODQw&quot;)\n \n \n \n pika |&gt; pika_gen(&quot;core&quot;)\n // -&gt; Error(&quot;Prefix is undefined&quot;)\n \nSynonyms:\npika_id.pika_gen\npika_id pika_gen","url":"pika_id.html#pika_gen"},{"doc":"pika_id","title":"pika_init","content":"pub fn pika_init(prefixes: List(Prefix)) -&gt; Result(Pika, String)\n Initialize Pika with default Epoch and Node ID.\n \n `Pika.gen(String) | pika_gen(Pika, String)` - generate ID\n \n `Pika.deconstruct(String) | pika_deconstruct(Pika, String)` - deconstruct ID\n \n To guarantee that developers use the correct pre-defined prefix types for the right object types, pika requires you to &quot;register&quot; them before they&#39;re used to prevent warnings from being thrown. \n \n This is also where you define if a prefix type should be cryptographically secure or not.\n \n ## Examples\n \n \n let assert Ok(pika) =\n   pika_init([\n     Prefix(&quot;user&quot;, &quot;User ID&quot;, False),\n     Prefix(&quot;sk&quot;, &quot;Secret key&quot;, True),\n   ]) \n // -&gt; Pika\n \n let assert Ok(pika) =\n   pika_init([\n     Prefix(&quot;u!ser&quot;, &quot;User ID&quot;, False),\n   ]) \n // -&gt; &quot;Unable to initialize pika: &quot;One of the prefixes is invalid (prefix &#39;u!ser&#39; must be Alphanumeric)&quot;&quot;\n \nSynonyms:\npika_id.pika_init\npika_id pika_init","url":"pika_id.html#pika_init"},{"doc":"pika_id","title":"pika_id","content":"","url":"pika_id.html"}]);